/**
 * @file sprmsched01/tasks.c
 *
 * @brief A heuristic example to demonstrate how the postponed jobs are handled.
 *
 * Given two tasks with implicit deadline. To force deadline misses, we reverse 
 * the rate-monotonic priority assignment and only execute the highest priority 
 * task twice. In the original implementation, no matter how many periods are 
 * expired, RMS manager only releases a job with a shifted deadline assignment 
 * in the watchdog.
 *
 * In more general real-time task model, e.g., soft real-time task, the postponed
 * jobs are still required to be released when the task is available to execute.
 */

/*
 *  COPYRIGHT (c) 2016 Kuan-Hsun Chen, TU Dortmund University (TUDo).
 *
 *  The license and distribution terms for this file may be
 *  found in the file LICENSE in this distribution or at
 *  http://www.rtems.com/license/LICENSE.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "system.h"

/* CPU usage and Rate monotonic manger statistics */
#include "rtems/cpuuse.h"

/* Periods for the various tasks [ticks] */
#define PERIOD_TASK_1          10000
#define PERIOD_TASK_2          2000

/* Task Id */
#define ID_TASK_1           0
#define ID_TASK_2          	1

/* Execution time for each task [ticks] */
#define task_1_normal_et	6000
#define task_2_normal_et	1000

/**
 * @brief Task 1 body
 */
rtems_task Task_1(
  rtems_task_argument unused
)
{
  rtems_status_code status;
  rtems_name        period_name; 
  rtems_id          RM_period;
  rtems_id          selfid=rtems_task_self();
  int               start, end;
  int		    	      tsk_counter = 0;

  /*create period*/
  period_name = rtems_build_name( 'P', 'E', 'R', '1' );
  status = rtems_rate_monotonic_create( period_name, &RM_period );
  if( RTEMS_SUCCESSFUL != status ) {
    printf("RM failed with status: %d\n", status);
    rtems_test_exit(0);
  }

	while( 1 ) {
		if(tsk_counter == 2){		
				status = rtems_rate_monotonic_delete(RM_period);
				if(status != RTEMS_SUCCESSFUL){
					printf("BUG: Cannot delete the period 1\n");
          rtems_test_exit(0);
				}
        
        status=rtems_task_delete(selfid);
        if(status != RTEMS_SUCCESSFUL){
          printf("BUG: Cannot delete the task 1\n");
          rtems_test_exit(0);
        }
        
    }
		status = rtems_rate_monotonic_period( RM_period,PERIOD_TASK_1);
    start = rtems_clock_get_ticks_since_boot();    
    printf("Task 1 starts at tick %d.\n", start);
    LOOP(task_1_normal_et,task_id);
    end = rtems_clock_get_ticks_since_boot();
    printf("					Task 1 ends at tick %d.\n", end);
    tsk_counter += 1;
	}
}

/**
 * @brief Task 2 body
 */
rtems_task Task_2(
  rtems_task_argument unused
)
{
  rtems_status_code status;
  rtems_name        period_name; 
  rtems_id          RM_period;
  int               start, end;
  int		            tsk_counter = 0;

  /*create period*/

  period_name = rtems_build_name( 'P', 'E', 'R', '2' );
  status = rtems_rate_monotonic_create( period_name, &RM_period );
  if( RTEMS_SUCCESSFUL != status ) {
    printf("RM failed with status: %d\n", status);
    exit( 0 );
  }

	while( 1 ) {
 	
		status = rtems_rate_monotonic_period( RM_period, PERIOD_TASK_2);
  	if(tsk_counter == testnumber){        
		  rtems_rate_monotonic_report_statistics();
			status = rtems_rate_monotonic_delete(RM_period);
			if(status != RTEMS_SUCCESSFUL){
				printf("BUG: Cannot delete the period 2\n");
        rtems_test_exit(0);
			}
			/* The example is finished */
      TEST_END();
      rtems_test_exit(0);
		}
    start = rtems_clock_get_ticks_since_boot();
    printf("Task 2 starts at tick %d.\n", start);
    LOOP(task_2_normal_et,task_id);
    end = rtems_clock_get_ticks_since_boot();
    printf("					Job %d Task 2 ends at tick %d.\n", tsk_counter+1, end);
    tsk_counter += 1;
	}
}

